/** 
 * @file   DFBiokey3000.h
 * @author Bernd Ottmann, Maik Siebrand
 * @date   05.06.2008
 * @brief  \deutsch Deklarationen der Exportroutinen.  \enddeutsch
           \english declaration of export functions.   \endenglish
 * @note   \deutsch  Die Auflistung der Routinen ist wie in der zugehoerigen Dokumentation. \enddeutsch 
           \english  the listing of functions is belong the documentation \endenglish
 
 */
//////////////////////////////////////////////////////////////////////
#ifndef _DFBIOKEY3000_H_
#define _DFBIOKEY3000_H_

/** 
 \deutsch @mainpage Dokumentation zur DFBiokey3000.dll. 
 \enddeutsch
 \english @mainpage Documentation of DFBiokey3000.dll.   
 \endenglish

 <center> @code 

   ########################################################
   ##       |                   |                  |     ##
   ##                          \_/                       ##
   ##       |                 -(_)-                |  \  ##
   ##                          / \                    |\\##
   ##  /|   |                                      |  || ##
   ##/ ||                  _\|||/_                    || ##
   ##  ||   |               /_=_\                  |  || ##
   ##  ||                  ({}-{})                    || ##
   ##  ||   |+-----oOo-------(_)---------------+   |  || ##
   ## //     |                                 |      || ##
   ##/      ||          Datafox GmbH           |   |  || ##
   ##        |                                 |      || ##
   ##       |+-------------------------oOo-----+   |  || ##
   ##                     |  |~|  |                   || ##
   ##       |_ __ _ __ _ _|__| |__|_ __ _ __ _ __ _|  || ##
   ##      /               ||   ||                  \ || ##
   ##    /               .ooO   Ooo.                  \ \##
   ##  /                                                \##
   ########################################################

 @endcode </center>
 @author Bernd Ottmann
 @author Maik Siebrand
 @date   19.08.2008
 @version 1.0.1.3
*/

/*

Version 1.0.0.1 vom 18.06.2008
Erstellung der Grundfunktionen

- DFBComOpen
- DFBGetLastErrorNum
- DFBComClose
- DFBInit
- DFBScanFinger 
- DFBGetTemplate

Version 1.0.1.1 vom 23.07.2008
Erweiterung um folgende Funktionen

- DFBDeleteTemplate
- DFBEnrollTemplate
- DFBFirmwareUpdate
- DFBGetSoftwareVersion
- DFBGetTemplateList
- DFBMatchTemplate
- DFBReadTemplate
- DFBSetTemplate 
- DFBWriteTemplate

Version 1.0.1.2 vom 30.07.2008
Keine Erweiterungen, nur Optimierungen des Programmes

Version 1.0.1.3 vom 19.08.2008
- Einarbeiten der FW-Version 6.0

Version 1.0.1.4 vom 08.12.2010
- ndern der Timeouts fr Biokey4000

Version 1.0.1.5 vom 10.12.2010
Erweiterung um folgende Funktionen

- DFBScanFingerSetCallBack


*/

// ########################################################
// ###################### DEFINES #########################
// ########################################################

/**
 * \deutsch Gibt export oder import der Routinen an. Siehe auch EXTERN_DECL. \enddeutsch
 *
 * @todo \deutsch Mit 1 im Quelltext vor #include zu definieren um die Routinen als Import zu erhalten. \enddeutsch
 *
 * @code #define AE3COM_EXPORT   1 @endcode
 */
#ifndef DLL_EXPORT
	#define DLL_EXPORT 0
#endif

#ifndef USE_PRIVATE
	#define USE_PRIVATE   0
#endif

// Eventuell fehlende typedefs ersetzen
#ifdef _WIN32

#ifndef HANDLE
	#define HANDLE void*
#endif

#endif
// ++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// +------------+ EXPORTIERTE FUNKTIONEN +----------------+
// ++++++++++++++++++++++++++++++++++++++++++++++++++++++++

/**
 @def EXTERN_DECL \deutsch Gibt den Praefix fuer Funktionen an. (Dekorierung und Export-, Importangabe) \enddeutsch
                  \english define the prefix of function \endenglish
*/

#ifdef _WIN32

#if (DLL_EXPORT)
	// Einzubindende Bibliotheken: Version.lib, WSock32.lib
  // please make a link to libraries :  Version.lib, WSock32.lib
	#define EXTERN_DECL         extern "C" __declspec(dllexport)
#else
	#define EXTERN_DECL         extern "C" __declspec(dllimport)
#endif

#else

#define EXTERN_DECL             extern "C"

#ifdef __declspec
	#undef __declspec
#endif

#define __declspec(dllexport)

#ifdef __stdcall
	#undef __stdcall
#endif

#define __stdcall

#endif

// ########################################################
// ############## ALLGEMEINE FUNKTIONEN ###################
// ##############  GENERAL   FUNCTIONS  ###################
// ########################################################



/*###########################################################*/
//  FUNCTION:	  DFBComOpen
/** \deutsch ffnet die RS232 \enddeutsch
    \english open the serial comport \endenglish

 @author Maik Siebrand
 @author Bernd Ottmann
 @date   25.10.2006

\deutsch   
@b Exportiert:
\enddeutsch
\english
@b Exported:
\endenglish
 _DFBComOpen@@4

@param[in] commStr \deutsch Sendestring COM1, COM2 ... \enddeutsch
                   \english param string COM1, COM2 ... \endenglish

@return \deutsch Erfolgsmeldung = TRUE, Fehlerfall = FALSE \enddeutsch
        \english no errors give TRUE, if errors occurred give FALSE \endenglish
\deutsch   
@b AUFGABE:    
- ffnet die RS232 falls mglich.
\enddeutsch

\english
@b JOB:
- Open the serial comport RS232 if possible.
\endenglish
	
\deutsch @b NDERUNG: \enddeutsch
\english @b MODIFICATIONS: \endenglish

- 06.06.2008 (MS) \deutsch bergabe String mit Schnittstellenbezeichnung \enddeutsch
                  \english param string with interface name \endenglish
  
		
@b TEST :     
- 25.10.2006 (MS) \deutsch keine Fehler festgestellt \enddeutsch \english no errors detected \endenglish
- 06.06.2008 (MS) \deutsch keine Fehler festgestellt \enddeutsch \english no errors detected \endenglish


\deutsch   
@b BEISPIEL:    
\enddeutsch

\english
@b EXAMPLE:
\endenglish

  if (!DFBComOpen("COM1")) error = DFBGetLastErrorNum();


*/  
/*###########################################################*/

EXTERN_DECL int __stdcall DFBComOpen(char *commStr);

/*###########################################################*/
//  FUNCTION:	  DFBGetLastErrorNum
/** \deutsch Gibt den letzten Fehlercode zurck \enddeutsch
    \english give the last error code \endenglish


@author Maik Siebrand
@date   06.06.2008

\deutsch   
@b Exportiert:
\enddeutsch
\english
@b Exported:
\endenglish
 _DFBGetLastErrorNum@@0

@return \deutsch Fehlercode \enddeutsch
        \english error code \endenglish
\deutsch
@b AUFGABE:    
- Gibt die Nummer des letzten Fehlers zurck.
\enddeutsch

\english
@b JOB:
- give the number of last occurred error
\endenglish

\deutsch @b Fehlercodes: \enddeutsch
\english @b error @b codes: \endenglish

\deutsch @b allgemeine \enddeutsch
\english @b general    \endenglish

-1 ComPort Open Error / \deutsch Fehler beim ffnen der Schnittstelle \enddeutsch
                        \english interface opening error  \endenglish
-2 ComPort Settings Error / \deutsch Biokey 3000 kann nicht erreicht werden \enddeutsch
                            \english the fingerprint module can not communicate \endenglish
-3 Communication Error / \deutsch Biokey 3000 antwortet nicht auf Anfrage \enddeutsch
                         \english no answer after send command \endenglish
-4 Scann Error / \deutsch Scann Fehler, Qualitt oder Minuzinanzahl zu schlecht \enddeutsch
                 \english quality oder minutiae count too less \endenglish
-5 Template Format Error / \deutsch Formatfehler beim Template \enddeutsch
                           \english wrong template format was using  \endenglish
-6 Template Buffer Size to small / \deutsch vorgegebener Templatepuffer ist zu klein \enddeutsch
                                   \english size of template buffer too small \endenglish
-7 empty file for update / \deutsch Datei fr Firmwareupdate ist leer \enddeutsch
                           \english updatefile is empty or could not open \endenglish
-8 memory error / \deutsch Speicherfehler \enddeutsch
                 \english a memory error is occurred \endenglish
-9 init module missing /   \deutsch das Biometriemodul wurde noch nicht initialisiert \enddeutsch
                           \english the functions DFBInit() was not started before \endenglish
-10 person not recognized / \deutsch Person wurde nicht erkannt bzw. Trefferqote nicht ausreichend \enddeutsch
                           \english the score rate of matching in too bad \endenglish

\deutsch @b modulspezifische \enddeutsch
\english @b device @b define @b errors \endenglish

130. no or bad image \deutsch / kein oder schlechtes Bild \enddeutsch @n
131. bad template \deutsch / schlechtes Template, nicht genug Merkmale \enddeutsch @n
132. database full \deutsch / Datenbank voll \enddeutsch @n
133. database empty \deutsch / Datenbank leer \enddeutsch @n
134. given IDs are used already \deutsch / uebergebene PIDs existieren nicht \enddeutsch @n
135. given IDs do not exist \deutsch / uebergebene PIDs existieren nicht \enddeutsch @n
138. image from sensor could not be acquired \deutsch / es wurde kein Bild vom Sensor aufgenommen \enddeutsch @n

@b TEST :     
- 06.06.2008 (MS) \deutsch keine Fehler festgestellt \enddeutsch \english no errors detected \endenglish
- 22.07.2008 (MS) \deutsch keine Fehler festgestellt \enddeutsch \english no errors detected \endenglish

\deutsch   
@b BEISPIEL:    
\enddeutsch

\english
@b EXAMPLE:
\endenglish

  if (!DFBComOpen("COM1")) error = DFBGetLastErrorNum();


*/
/*###########################################################*/

EXTERN_DECL int __stdcall DFBGetLastErrorNum(void);

/*###########################################################*/
//  FUNCTION:	  DFBGetLastErrorText
/** \deutsch Gibt zum Fehlercode den Fehlertext zurck \enddeutsch
    \english give the text of error code \endenglish 

@author Maik Siebrand
@date   22.07.2008

\deutsch   
Exportiert:
\enddeutsch
\english
exported:
\endenglish
 _DFBGetLastErrorText@@12

@param[in] ErrNum \deutsch Nummer des letzten Fehlers zurck \enddeutsch
@param[out] ErrText \deutsch Fehlerbeschreibung in englischer Sprache \enddeutsch
@param[in] TextBufLen \deutsch Groesze des Puffers von ErrText \enddeutsch

\deutsch
@b AUFGABE:    
- Gibt die Beschreibung des letzten Fehlers zurck.
- Ist die Beschreibung groeszer als der Puffer wird die 
  Beschreibung abgeschnitten.

@b Fehlercodes: 

@b allgemeine

-# ComPort Open Error / Fehler beim ffnen der Schnittstelle
-# ComPort Settings Error / Biokey 3000 kann nicht erreicht werden
-# Communication Error / Biokey 3000 antwortet nicht auf Anfrage
-# Scann Error / Scann Fehler, Qualitt oder Minuzinanzahl zu schlecht
-# Template Format Error / Formatfehler beim Template
-# Template Buffer Size to small / vorgegebener Templatepuffer ist zu klein
-# empty file for update / Datei fr Firmwareupdate ist leer
-# memory error / Speicherfehler
-# init module missing / das Biometriemodul wurde noch nicht initialisiert
-# person not recognized / Person wurde nicht erkannt bzw. Trefferqote nicht ausreichend  

@b modulspezifische 

130. no or bad image / kein oder schlechtes Bild  @n
131. bad template / schlechtes Template, nicht genug Merkmale  @n
132. database full / Datenbank voll  @n
133. database empty / Datenbank leer  @n
134. given IDs are used already / uebergebene PIDs existieren nicht  @n
135. given IDs do not exist / uebergebene PIDs existieren nicht  @n
138. image from sensor could not be acquired / es wurde kein Bild vom Sensor aufgenommen @n
\enddeutsch

  \english
@b JOB:
- give the description of error number back
- if the description bigger than the buffer, the text will cut to max. size
\endenglish


@b TEST :     
- 22.07.2008 (MS) \deutsch keine Fehler festgestellt \enddeutsch \english no errors detected \endenglish

\deutsch   
@b BEISPIEL:    
\enddeutsch

\english
@b EXAMPLE:
\endenglish

  int error = 0;       @n
  char buffer[50];     @n

  if (!DFBComOpen("COM1")) {    @n

    error = DFBGetLastErrorNum();
  
    DFBGetLastErrorText(error, buffer, sizeof(buffer));

  }


*/
/*###########################################################*/

EXTERN_DECL void __stdcall DFBGetLastErrorText(int ErrNum,char *ErrText,int *TextBufLen);

/*###########################################################*/
//  FUNCTION:	  DFBComClose
/** \deutsch Schliet die RS232 \enddeutsch
    \english close the serial comport \endenglish

@author Maik Siebrand
@date   25.10.2006

\deutsch   
@b Exportiert:
\enddeutsch
\english
@b Exported:
\endenglish
 _DFBComClose@@0

\deutsch   
@b AUFGABE:    
- Schliet die RS232
\enddeutsch

\english
@b JOB:
- Close the serial comport RS232
\endenglish


@b TEST :     
- 25.10.2006 (MS) \deutsch keine Fehler festgestellt \enddeutsch \english no errors detected \endenglish
- 06.06.2008 (MS) \deutsch keine Fehler festgestellt \enddeutsch \english no errors detected \endenglish


\deutsch   
@b BEISPIEL:    
\enddeutsch

\english
@b EXAMPLE:
\endenglish

  DFBComClose();


*/  
/*###########################################################*/

EXTERN_DECL void __stdcall DFBComClose();

/*############################################################################*/
//  FUNCTION:	 DFBInit
/** \deutsch Initialisierung des BioKey3000 \enddeutsch
    \english initialization of Biokey3000 \endenglish

@author Maik Siebrand
@date   06.06.2008

\deutsch   
@b Exportiert:
\enddeutsch
\english
@b Exported:
\endenglish
 _DFBInit@@16

@param[in] TemplateFormat 0 DIN-Format, 1 Idencom Compact Format
@param[in] SecurityLevel \deutsch Sicherheitslevel, das sich aus Falschrueckweisungrate und Falschakzeptanzrate ergibt \enddeutsch
                         \english security level is the threshold for recognizing of matching score \endenglish
@param[in] Quality \deutsch Mindestqualitt der Templates \enddeutsch
                   \english min. value of quality \endenglish
@param[in] MinutiaeMin \deutsch Mindestanzahl der Minuzien \enddeutsch
                       \english min. value of minutiae count \endenglish

@return \deutsch Erfolgsmeldung = TRUE, Fehlerfall = FALSE \enddeutsch
        \english no errors give TRUE, if errors occurred give FALSE \endenglish

\deutsch   
@b AUFGABE:    
- Sendet die wichtigsten Einstellungen aus dem Setup zum BioKey.
\enddeutsch

\english
@b JOB:
- send the setting of configuration to the biokey3000
\endenglish


@b TEST :
- 06.06.2008 (MS) \deutsch keine Fehler festgestellt \enddeutsch \english no errors detected \endenglish
- 30.07.2008 (MS) \deutsch keine Fehler festgestellt \enddeutsch \english no errors detected \endenglish

\deutsch   
@b BEISPIEL:    
\enddeutsch

\english
@b EXAMPLE:
\endenglish

  unsigned char TemplateFormat=1;  @n
  unsigned char SecurityLevel=50;  @n
  unsigned char Quality=40;        @n
  unsigned char MinutiaeMin=12;    @n

  if (!DFBInit(TemplateFormat,SecurityLevel,Quality,MinutiaeMin)) error = DFBGetLastErrorNum();


*/
/*############################################################################*/

EXTERN_DECL int __stdcall DFBInit(unsigned char TemplateFormat, unsigned char SecurityLevel, unsigned char Quality, unsigned char MinutiaeMin);

/*###########################################################*/
//  FUNCTION:	  DFBScanFinger
/** \deutsch Scannt einen Finger in TemplateBufferA(Sensorbuffer FTP_BUF) \enddeutsch
    \english get a template from scanner and store it in TemplateBufferA(Sensorbuffer FTP_BUF) \endenglish

@author Maik Siebrand
@date   06.06.2008

\deutsch   
@b Exportiert:
\enddeutsch
\english
@b Exported:
\endenglish
 _DFBScanFinger@@4

@param[in] BestScann \deutsch 0..1 fr einen Scann 2..10 Anzahl der Scanversuche fr den besten Fingerscann \enddeutsch
                     \english 0 or 1 for one scann and  2..10 count of scanns for the best scann \endenglish

@return \deutsch Erfolgsmeldung = TRUE, Fehlerfall = FALSE \enddeutsch
        \english no errors give TRUE, if errors occurred give FALSE \endenglish
\deutsch
@b AUFGABE:    
- Scannt einen Finger ein und gibt wenn die Qualitt ausreichend war ein OK zurck. 
  Die Funktion kann durch Drcken der ESC-Taste abgebrochen werden.
- Die BestScann Variable gibt an ob aus mehreren Scanns das beste Template nach 
  Beendigung der Funktion im Templatepuffer zurckgehalten wird.
\enddeutsch

  \english
@b JOB:
- make a scann of a finger and give back an OK if the quality good enough.
- the variable BestScann you can define how muchs scanns are needed to store the 
  best template in the TemplateBufferA.
\endenglish


@b TEST :
- 06.06.2008 (MS) \deutsch keine Fehler festgestellt \enddeutsch \english no errors detected \endenglish

\deutsch   
@b BEISPIEL:    
\enddeutsch

\english
@b EXAMPLE:
\endenglish


  memset(g_TemplateA,0,sizeof(g_TemplateA));

  if (!DFBComOpen("COM1")) error = DFBGetLastErrorNum();

  if (!DFBInit(1,50,40,12)) error = DFBGetLastErrorNum();

  if (!DFBScanFinger(1)) error = DFBGetLastErrorNum();

  size=sizeof(g_TemplateA);

  if (!DFBGetTemplate(&size,g_TemplateA)) error = DFBGetLastErrorNum();

  @note \deutsch jetzt kann man das Template in einer Datei speichern \enddeutsch
        \english now you can save the template in a file. \endenglish


*/
/*###########################################################*/

EXTERN_DECL int __stdcall DFBScanFinger(unsigned char BestScann);

/*###########################################################*/
//  FUNCTION:	  DFBGetTemplate
/** \deutsch liest den TemplateBufferA(Sensorbuffer FTP_BUF) aus \enddeutsch
    \english get a template from TemplateBufferA(Sensorbuffer FTP_BUF) \endenglish

@author Maik Siebrand
@date   06.06.2008

\deutsch   
@b Exportiert:
\enddeutsch
\english
@b Exported:
\endenglish
 _DFBGetTemplate@@8

@param[in] BufferSize \deutsch Gre des im System vorgehaltenen Speichers fr das Template \enddeutsch
                      \english size of Buffer in the memory \endenglish
@param[out] Buffer \deutsch Templatepuffer 161Byte fr DIN-Format und 216 Byte fr Idencom Compact Format \enddeutsch
                   \english Buffer in the memory for store the template, 161byte for the DIN-Format and 216byte for the Idencom-Compact-Format \endenglish
@return \deutsch Erfolgsmeldung = TRUE, Fehlerfall = FALSE \enddeutsch
        \english no errors give TRUE, if errors occurred give FALSE \endenglish
\deutsch
@b AUFGABE:    
- Liest einen Finger aus dem Templatebuffer A aus und prft vorher die Puffergre ab.               
\enddeutsch

\english
@b JOB:
- read a template from TemplateBufferA and do check the buffersize
\endenglish

@b TEST :
- 06.06.2008 (MS) \deutsch keine Fehler festgestellt \enddeutsch \english no errors detected \endenglish

\deutsch   
@b BEISPIEL:    
\enddeutsch

\english
@b EXAMPLE:
\endenglish


  memset(g_TemplateA,0,sizeof(g_TemplateA));

  if (!DFBComOpen("COM1")) error = DFBGetLastErrorNum();

  if (!DFBInit(1,50,40,12)) error = DFBGetLastErrorNum();

  if (!DFBScanFinger(1)) error = DFBGetLastErrorNum();

  size=sizeof(g_TemplateA);

  if (!DFBGetTemplate(&size,g_TemplateA)) error = DFBGetLastErrorNum();

  @note \deutsch jetzt kann man das Template in einer Datei speichern \enddeutsch
        \english now you can save the template in a file. \endenglish


*/
/*###########################################################*/

EXTERN_DECL int __stdcall DFBGetTemplate(unsigned int *BufferSize, unsigned char *Buffer);

/*###########################################################*/
//  FUNCTION:	  DFBSetTemplate
/** \deutsch bertrgt ein Template in das Biokeymodul  \enddeutsch
    \english transmit a template to module \endenglish

@author Maik Siebrand
@date   10.06.2008

\deutsch   
@b Exportiert:
\enddeutsch
\english
@b Exported:
\endenglish
 _DFBSetTemplate@@8

@param[in] TemplateBuffer \deutsch Zeiger auf den Speicher wo die Templatedaten liegen \enddeutsch
           \english pointer of template in the memory \endenglish
\deutsch
@param[in] Destination Zielspeicher fr das Template @n 
                       Templatebuffer A - 0 (Sensorpuffer), @n
                       Templatebuffer B - 1, @n
                       Templatebuffer C - 2  @n 
\enddeutsch
\english
@param[in] Destination memory area in the module @n
                       Templatebuffer A - 0 (sensor buffer), @n
                       Templatebuffer B - 1, @n
                       Templatebuffer C - 2  @n  
\endenglish

@return \deutsch Erfolgsmeldung = TRUE, Fehlerfall = FALSE \enddeutsch
        \english no errors give TRUE, if errors occurred give FALSE \endenglish

\deutsch  
@b AUFGABE:    
- bertrgt das in TemplateBuffer abgelegte Template in das unter Destination angegebene Ziel im Biokey3000.
\enddeutsch

\english
@b JOB:
- transmit the template in TemplateBuffer to Destination in the module
\endenglish

\deutsch  
@b KOMMENTARE: Die Aufteilung des Templates ist folgendermaen

- Kopfdaten Fix 11 Byte
  - PID - 8Byte  / die Datafox Terminals benutzen nur 4 Byte
  - FID - 1Byte
  - Quality - 1Byte
  - Anzahl Minutiae im Template - 1Byte
  - Datensaetze Minutiae

- Datensatz DIN-Format - Minutiaedaten Fix 3 Byte, max. Anzahl 50 
  - X-Position - 1Byte
  - Y-Position - 1Byte
  - W-Winkel - 1Byte

- Datensatz Idencom Compact Format - Minutiaedaten Fix 5 Byte, max. Anzahl 41  
  - X-Position - 1Byte
  - Y-Position - 1Byte
  - W-Winkel - 1Byte
  - X-Position Nachbar- 1Byte
  - Y-Position Nachbar - 1Byte
\enddeutsch

\english  
@b COMMENTARY : the struct of templates is shown you follow

- header fix 11 byte
  - PID(personal identification number) - 8byte / the datafox terminals using only 4 byte
  - FID(finger identification number) - 1byte
  - quality - 1byte
  - minutiae count of template - 1byte
  - records of minutiae

- RECORD DIN-Format - size of minutiae data is fix 3 byte, max. count 50 
  - X-position - 1byte
  - Y-position - 1byte
  - W-angle - 1byte

- RECORD Idencom Compact Format - size of minutiae data is fix 5 byte, max. count 41
  - X-position - 1byte
  - Y-position - 1byte
  - W-angle - 1byte
  - X-position neighbour - 1byte
  - Y-position neighbour - 1byte
\endenglish

		
@b TEST :     
- 22.07.2008 (MS) \deutsch keine Fehler festgestellt \enddeutsch \english no errors detected \endenglish

\deutsch   
@b BEISPIEL:    

  Schreiben des Templates in das Modul und Auslesen des Templates mit anschlieendem Vergleichen 
  ob das Template in das Modul korrekt bertragen wurde.
  Das sollte man whrend der Entwicklung als Kontrolle machen, spter wenn die Anwendung luft ist das nicht mehr notwendig.

\enddeutsch

\english
@b EXAMPLE:

  Write a template in the modul and read and compare it if the transfer was correct.
  You should do this during the software development as check, later you dont need this.

\endenglish


  if (!DFBSetTemplate((unsigned char*)&g_TemplateA,  0)) error = DFBGetLastErrorNum();

  size=sizeof(g_TemplateA);
  
  if (!DFBGetTemplate(&size,g_TemplateB)) error = DFBGetLastErrorNum();

  Result = memcmp(g_TemplateA,g_TemplateB,161);

  @note \deutsch wenn Result NULL ist, sind beide gleich und die bertragung erfolgte ohne Fehler \enddeutsch
        \english if Result NULL then was the transfer without errors \endenglish


*/  
/*###########################################################*/

EXTERN_DECL int __stdcall DFBSetTemplate(unsigned char *TemplateBuffer, unsigned char Destination);

/*###########################################################*/
//  FUNCTION:	  DFBMatchTemplate
/** \deutsch Fhrt einen Vergleich zweier Templates durch  \enddeutsch
    \english do match a template with another \endenglish

@author Maik Siebrand
@date   11.07.2008

\deutsch   
@b Exportiert:
\enddeutsch
\english
@b Exported:
\endenglish
 _DFBMatchTemplate@@20

\deutsch
@param[in] Source1 Sensor - 0 / Templatepuffer A - 1
@param[in] Source2 Datenbank - 0 / Templatepuffer B - 1 / Templatepuffer C - 2
@param[out] Score Trefferqote in Prozent 0..99
@param[out] PID 64bit Wert fr die Persnliche Identifizierungsnummer "unsigned char PID[8]"
@param[out] FID 8bit Wert fr die Fingernummer 
\enddeutsch
\english
@param[in] Source1 sensor - 0 / template buffer A - 1
@param[in] Source2 data base of module - 0 / template buffer B - 1 / template buffer C - 2
@param[out] Score matching score in per cent 0..99
@param[out] PID personal identification number - 8byte / the datafox terminals using only 4 byte
@param[out] FID finger identification number - 1byte
\endenglish

@return \deutsch Erfolgsmeldung = TRUE, Fehlerfall = FALSE \enddeutsch
        \english no errors give TRUE, if errors occurred give FALSE \endenglish
\deutsch  
@b AUFGABE:    
- Vergleicht zwei Templates miteinander und gibt die Trefferqote zurck.
- Bei Identifizierung(Matching Sensor mit Datenbank) wird die PID und 
  FID mit zurckgegeben.
- Bei Verifizierung(Matching Sensor mit Datenbank) muss PID(z.B. Ausweisnummer) 
  und FID(kann auch 0 sein fr alle Finger der PID) mit bergeben werden   
\enddeutsch
\english
@b JOB:
- machting templates and give the matching score
- matching against the database give also PID and FID
  - start the function with PID = 0 and FID = 0 will match with entire database
  - start the function with PID > 0 and FID = 0 will match with all templates with PID
  - start the function with PID > 0 and FID > 0 will match only templates with PID and FID
\endenglish

\deutsch
@b KOMMENTARE:
- zum Vergleichen zweier Templates, die im PC vorhanden sind, 
  muss ein Template in Templatepuffer A und das andere in 
  Templatepuffer B oder C bertragen werden. Anschlieend muss 
  dieser Befehl mit Source1 = 1 und Source2 = 1 oder 2 aufgerufen 
  werden.
- zum Identifizieren/Test von Templates, kann das Modul auch herangezogen 
  werden, wenn alle Templates in der Datenbank des Biokey3000 abgelegt sind.
  In diesem Fall mssen Source1 und Source2 Null sein. Bei der Identifizierung 
  mssen PID und FID mit Null bergeben werden. Bei einer Verifizierung mssen 
  die entsprechenden Werte vorgegeben werden.
- Fehlerfaelle die DFBGetLastErrorNum() liefert
  -# (0x03) Kommunikationsfehler
  -# (0x82) kein oder schlechtes Bild
  -# (0x83) schlechtes Template, nicht genug Merkmale
  -# (0x85) Datenbank leer
  -# (0x87) uebergebene PIDs existieren nicht
\enddeutsch
\english  
@b COMMENTARY: 
- To match two template witch are available on the computer you have tranfer one 
  template to template buffer A and the other in template buffer B. 
  After this call this function with Source1 = 1 and Source2 = 1.
- error cases could be
  -# (0x03) Communication Error
  -# (0x82) no or bad image
  -# (0x83) bad template
  -# (0x85) database empty
  -# (0x87) given IDs do not exist
\endenglish
  
@b TEST:     
- 22.07.2008 (MS) \deutsch keine Fehler festgestellt \enddeutsch \english no errors detected \endenglish

\deutsch   
@b BEISPIEL:    

bertragen eines Templates in TemplatebufferA des Fingerprintmoduls und bertragen eines anderen 
Templates in TemplatebufferB des Fingerprintmoduls. Starten des Matchvorganges von TemplatebufferA 
mit TemplatebufferB. 

\enddeutsch

\english
@b EXAMPLE:

Tranfer a template in template buffer A of BIOKEY3000 and tranfer another template template 
buffer B of BIOKEY3000. Start matching with template buffer A and B. 

\endenglish

          error = 0; @n
          PID = 0;   @n
          FID = 0;   @n          
          
          if (!DFBSetTemplate  ( TemplateA ,  0)) error = DFBGetLastErrorNum();
          
          if (!DFBSetTemplate  ( TemplateB,  1)) error = DFBGetLastErrorNum();
          
          if (!DFBMatchTemplate  ( 1,  1, &Score , (unsigned char*)&PID, &FID)) 
          {

            error = DFBGetLastErrorNum();

            switch (error)
            {
               
               case 0x0A: //person not recognized @n
               break; @n
               case 0x03: //communication error @n
               break;@n
            }   @n
          }     @n
          else
          {
            ... Code for person recognized
          }

  @note \deutsch in Score ist die Trefferqote des Vergleiches als Messgre fr die Erkennung \enddeutsch
        \english in Score is the matching score as measure value for recognizing \endenglish

*/  
/*###########################################################*/

EXTERN_DECL int __stdcall DFBMatchTemplate(unsigned char Source1, unsigned char Source2, unsigned char *Score, unsigned char *PID, unsigned char *FID);

/*###########################################################*/
//  FUNCTION:	  DFBEnrollTemplate
/** \deutsch Einlernen eines Templates   \enddeutsch
    \english enroll a scann from sensor \endenglish

@author Maik Siebrand
@date   11.07.2008

\deutsch   
@b Exportiert:
\enddeutsch
\english
@b Exported:
\endenglish
 _DFBEnrollTemplate@@20

\deutsch
@param[in] PID Personenidentittsnummer unter der das Template abgelegt werden soll, meist 32Bit->4Byte, max. 64Bit->8Byte gro 
@param[in] PIDLength Lnge Personenidentittsnummer (max. 8) oder 0 fr 64Bit-Wert von PID
@param[in] FID Fingernummer, wenn Null sucht das Modul alle Werte sonst nur den Wert in FID
@param[in] Source Datenquelle Sensor = 0 / TemplatepufferA = 1 
@param[in] Destination  wohin mit den Daten Datenbank+TemplatepufferB = 0, TemplatepufferB= 1, TemplatepufferC= 2
\enddeutsch
\english
@param[in] PID personal identification number - 8byte / the datafox terminals using only 4 byte
@param[in] PIDLength length of PID in byte (max. 8) or 0 for 64bit-value of PID
@param[in] FID finger identification number - 1byte
@param[in] Source sensor - 0 / template buffer A - 1
@param[in] Destination  for template data id database of module - 0 / template buffer B - 1 / template buffer C - 2
\endenglish

@return \deutsch Erfolgsmeldung = TRUE, Fehlerfall = FALSE \enddeutsch
        \english no errors give TRUE, if errors occurred give FALSE \endenglish

\deutsch
@b AUFGABE:    
 - Einlernen eines Templates, diese Funktion veranlasst das Modul dazu 
   ein Bild vom Sensor zu lesen, daraus die Merkmale zu extrahieren 
   und das gewonnene Template(Fingerabdruck) abhngig von der 
   uebergebenen PID in der Datenbank des Moduls zu speichern.
\enddeutsch

\english
@b JOB:
- enroll a scann from sensor or template buffer to the destination database of module or a template buffer
\endenglish

\deutsch
@b KOMMENTARE: 
- Wenn PID und FID Null sind wird automatisch ein Wert dafr
  generiert, ansonsten wird der vorgegebene benutzt, falls er nicht
  schon verwendet wurde. In diesemm Fall wird ein Fehler zurckgegeben.

- Fehlerfaelle die DFBGetLastErrorNum() liefert
  -# (0x03) Kommunikationsfehler
  -# (0x82) unerwarteter Fehler aufgetreten
  -# (0x82) kein oder schlechtes Bild
  -# (0x83) schlechtes Template, nicht genug Merkmale
  -# (0x84) Datenbank voll
  -# (0x86) uebergebene PIDs werden schon benutzt
  -# (0x8A) es wurde kein Bild vom Sensor aufgenommen
\enddeutsch
\english  
@b COMMENTARY: 
- If PID and FID are NULL the module will make automaticly generate the numbers
- error cases could be
  -# (0x03) Communication Error
  -# (0x82) no or bad image
  -# (0x83) bad template
  -# (0x84) database full
  -# (0x86) given IDs already used
  -# (0x8A) image from sensor could not be acquired
\endenglish

@b TEST :     
- 22.07.2008 (MS) \deutsch keine Fehler festgestellt \enddeutsch \english no errors detected \endenglish

*/
/*###########################################################*/

EXTERN_DECL int __stdcall DFBEnrollTemplate(unsigned char *PID, unsigned char PIDLength, unsigned char FID, unsigned char Source, unsigned char Destination);

/*###########################################################*/
//  FUNCTION:	  DFBDeleteTemplate
/** \deutsch Lsche PID vom Modul \enddeutsch
    \english delete a PID from module \endenglish

@author Maik Siebrand
@date   11.07.2008

\deutsch   
@b Exportiert:
\enddeutsch
\english
@b Exported:
\endenglish
 _DFBDeleteTemplate@@12

\deutsch
@param[in] PID Personenidentittsnummer max 8 Bit, wenn Null sucht das Modul alle Werte sonst nur den Wert in PID
@param[in] PIDLength Lnge Personenidentittsnummer (max. 8) oder 0 fr 64Bit-Wert von PID
@param[in] FID Fingernummer, wenn Null sucht das Modul alle Werte sonst nur den Wert in FID
\enddeutsch

\english
@param[in] PID personal identification number - 8byte / the datafox terminals using only 4 byte
@param[in] PIDLength length of PID in byte (max. 8) or 0 for 64bit-value of PID
@param[in] FID finger identification number - 1byte
\endenglish

@return \deutsch Erfolgsmeldung = TRUE, Fehlerfall = FALSE \enddeutsch
        \english no errors give TRUE, if errors occurred give FALSE \endenglish

\deutsch
@b AUFGABE:    
- Lscht Finger aus der Datenbank/Flash. Diese Funktion veranlasst das Modul dazu ein 
  oder mehrere Templates aus der Datenbank zu lschen.
\enddeutsch

\english
@b JOB:
- Delete a Template or Templates of a PID in the database of module.
\endenglish

\deutsch
@b KOMMENTARE: 
- Wenn PID und FID Null sind, wird die gesamte Datenbank gelscht.
\enddeutsch
\english  
@b COMMENTARY: 
- If PID and FID are NULL the module will delete all templates in the module.
\endenglish

@b TEST :     
- 22.07.2008 (MS) \deutsch keine Fehler festgestellt \enddeutsch \english no errors detected \endenglish

*/
/*###########################################################*/

EXTERN_DECL int __stdcall DFBDeleteTemplate(unsigned char *PID, unsigned char PIDLength, unsigned char FID);

/*###########################################################*/
//  FUNCTION:	  DFBGetTemplateList
/** \deutsch Macht eine Listendatei mit den Biokey PIDs \enddeutsch
    \english Create a Listfile with the used PIDs of Module \endenglish


@author Maik Siebrand
@date   12.06.2008

\deutsch   
@b Exportiert:
\enddeutsch
\english
@b Exported:
\endenglish
 _DFBGetTemplateList@@8

@param[in] ListFilename \deutsch Dateiname und Pfad der Ausgabedatei \enddeutsch
                        \english name and path of file \endenglish
@param[out] ListEntries \deutsch Anzahl der Datenstze \enddeutsch
                        \english count of records in the file \endenglish

@return \deutsch Erfolgsmeldung = TRUE, Fehlerfall = FALSE \enddeutsch
        \english no errors give TRUE, if errors occurred give FALSE \endenglish

\deutsch
@b AUFGABE:    
- Liest die PIDs aus uns speichert sie in ListBuffer als unsortierte Liste. 

@b KOMMENTARE:
               
- Die Liste hat folgenden Aufbau:

  - PID=8Byte als Text 21Byte mit Tab
  - FID=3Byte als Text 5Byte mit CR und LF
  - Datensatzbreite ist also 28Byte
\enddeutsch

\english
@b JOB:
- Read the used PIDs of module and save it in a file, data are not sort

@b COMMENTARY: 
- the list has follow structure
  - PID as string, fieldsize 21byte with termination tabulator
  - FID as string, fieldsize 5byte with termination CR and LF
  - Recordsize is 28byte 

\endenglish


@b TEST :     
- 22.07.2008 (MS) \deutsch keine Fehler festgestellt \enddeutsch \english no errors detected \endenglish

*/
/*###########################################################*/

EXTERN_DECL int __stdcall DFBGetTemplateList( char *ListFilename, unsigned int *ListEntries);

/*############################################################################*/
//  FUNCTION:  DFBGetSoftwareVersion
/** \deutsch Ermittelt den Firmwarestand PIDs \enddeutsch
    \english get the module software version \endenglish


@author Maik Siebrand
@date   12.06.2008

\deutsch   
@b Exportiert:
\enddeutsch
\english
@b Exported:
\endenglish
 _DFBGetSoftwareVersion@@4

@param[out] Version \deutsch 4 stelliger Versionsstring\enddeutsch
                    \english 4 byte string of version \endenglish

@return \deutsch Erfolgsmeldung = TRUE, Fehlerfall = FALSE \enddeutsch
        \english no errors give TRUE, if errors occurred give FALSE \endenglish

\deutsch
@b Aufgaben:
- ermittelt den aktuellen Softwareversionsstand des Biokeys, vorausgesetzt die
  Funktion DFBInit() wurde vorher ausgefhrt.

@b Kommentare:
- Die Softwareversion ist in zwei Bereiche aufgeteilt, Version Firmware und 
  Version Scann-Engine @n
  1. Zeichen Firmwareversion Major @n
  2. Zeichen Firmwareversion Minor @n
  3. Zeichen Version Scann-Engine Major @n
  4. Zeichen Version Scann-Engine Minor @n

  Beispiel 5351 ist Firmware-Version 5.3 mit Scann-Engine Version 5.1
\enddeutsch

\english
@b JOB:
- get the used softwareversion of biokey3000 if you run before DFBInit().

@b COMMENTARY: 
- the softwareversion has two areas, version of firmware and version of scann engine @n
  1. character Firmwareversion Major @n
  2. character Firmwareversion Minor @n
  3. character Version Scann-Engine Major @n
  4. character Version Scann-Engine Minor @n

 Example 5351 is  firmware version 5.3 with scann engine version 5.1
\endenglish


@b Test:
- 03.03.2008 (MS) \deutsch keine Fehler festgestellt \enddeutsch \english no errors detected \endenglish
- 22.07.2008 (MS) \deutsch keine Fehler festgestellt \enddeutsch \english no errors detected \endenglish

*/
/*############################################################################*/

EXTERN_DECL int __stdcall DFBGetSoftwareVersion(char *Version);

/*###########################################################*/
//  FUNCTION:	  DFBWriteTemplate
/** \deutsch Schreibt die Fingerdaten(Template) in den BIOKEY3000 Flash \enddeutsch
    \english Write a template in the module database (flash memory) \endenglish

@author Maik Siebrand
@date   14.07.2008

\deutsch   
@b Exportiert:
\enddeutsch
\english
@b Exported:
\endenglish
 _DFBWriteTemplate@@16

\deutsch
@param[out] PID Persnliche Identifikationsnummer max. 8Byte bzw. 64Bit Integer Wert
@param[out] FID Fingernummer 1..10
@param[in] Buffer Daten des Templates, Aufteilung unter Kommentare
@param[in] BufferLength Buffer-Gre in Byte
\enddeutsch

\english
@param[out] PID personal identification number - 8byte / the datafox terminals using only 4 byte
@param[out] FID finger identification number - 1byte  1..10
@param[in] Buffer pointer of template data
@param[in] BufferLength size of buffer in byte
\endenglish

@return \deutsch Erfolgsmeldung = TRUE, Fehlerfall = FALSE \enddeutsch
        \english no errors give TRUE, if errors occurred give FALSE \endenglish

\deutsch
@b AUFGABE:    
- Schreibt den Finger aus dem Buffer mit der PID und FID in die Datenbank des Biokey3000.
\enddeutsch

\english
@b JOB:
- Write the template in database of module.
\endenglish

\deutsch  
@b KOMMENTARE: Die Aufteilung des Templates ist folgendermaen

- Kopfdaten Fix 11 Byte
  - PID - 8Byte  / die Datafox Terminals benutzen nur 4 Byte
  - FID - 1Byte
  - Quality - 1Byte
  - Anzahl Minutiae im Template - 1Byte
  - Datensaetze Minutiae

- Datensatz DIN-Format - Minutiaedaten Fix 3 Byte, max. Anzahl 50 
  - X-Position - 1Byte
  - Y-Position - 1Byte
  - W-Winkel - 1Byte

- Datensatz Idencom Compact Format - Minutiaedaten Fix 5 Byte, max. Anzahl 41  
  - X-Position - 1Byte
  - Y-Position - 1Byte
  - W-Winkel - 1Byte
  - X-Position Nachbar- 1Byte
  - Y-Position Nachbar - 1Byte
\enddeutsch

\english  
@b COMMENTARY : the struct of templates is shown you follow

- header fix 11 byte
  - PID(personal identification number) - 8byte / the datafox terminals using only 4 byte
  - FID(finger identification number) - 1byte
  - quality - 1byte
  - minutiae count of template - 1byte
  - records of minutiae

- RECORD DIN-Format - size of minutiae data is fix 3 byte, max. count 50 
  - X-position - 1byte
  - Y-position - 1byte
  - W-angle - 1byte

- RECORD Idencom Compact Format - size of minutiae data is fix 5 byte, max. count 41
  - X-position - 1byte
  - Y-position - 1byte
  - W-angle - 1byte
  - X-position neighbour - 1byte
  - Y-position neighbour - 1byte
\endenglish


@b TEST :     
- 22.07.2008 (MS) \deutsch keine Fehler festgestellt \enddeutsch \english no errors detected \endenglish

*/
/*###########################################################*/

EXTERN_DECL int __stdcall DFBWriteTemplate(unsigned char *PID, unsigned char *FID, unsigned char *Buffer, unsigned int *BufferLength);

/*###########################################################*/
//  FUNCTION:	  DFBReadTemplate
/** \deutsch Liest die Fingerdaten(Template) aus dem BIOKEY3000 Flash \enddeutsch
    \english Read a template from the module database (flash memory) \endenglish

@author Maik Siebrand
@date   14.07.2008

\deutsch   
@b Exportiert:
\enddeutsch
\english
@b Exported:
\endenglish
 _DFBReadTemplate@@20

\deutsch
@param[in] PID Persnliche Identifikationsnummer max. 8Byte bzw. 64Bit Integer Wert
@param[in] PIDLength Anzahl der bergebenen Bytes mit Daten (max. 8) bei 64Bit Integer Wert NULL
@param[in] FID Fingernummer 1..10
@param[out] Buffer Daten des Templates, Aufteilung unter Kommentare
@param[out] BufferLength Buffer-Gre in Byte
\enddeutsch

\english
@param[in] PID personal identification number - 8byte / the datafox terminals using only 4 byte
@param[in] PIDLength length of PID in byte (max. 8) or 0 for 64bit-value of PID
@param[in] FID finger identification number - 1byte  1..10
@param[out] Buffer pointer of template data
@param[out] BufferLength size of buffer in byte
\endenglish

@return \deutsch Erfolgsmeldung = TRUE, Fehlerfall = FALSE \enddeutsch
        \english no errors give TRUE, if errors occurred give FALSE \endenglish

\deutsch
@b AUFGABE:    
- Liest den Finger mit der PID aus dem BioKey-Modul und bergibt ihn in Buffer. 
- PID kann eine Zeichenkette sein oder ein 64Bit Integer Wert.
\enddeutsch

\english
@b JOB:
- Read a template with PID and FID from the database of module to Buffer. 
\endenglish

\deutsch  
@b KOMMENTARE: Die Aufteilung des Templates ist folgendermaen

- Kopfdaten Fix 11 Byte
  - PID - 8Byte  / die Datafox Terminals benutzen nur 4 Byte
  - FID - 1Byte
  - Quality - 1Byte
  - Anzahl Minutiae im Template - 1Byte
  - Datensaetze Minutiae

- Datensatz DIN-Format - Minutiaedaten Fix 3 Byte, max. Anzahl 50 
  - X-Position - 1Byte
  - Y-Position - 1Byte
  - W-Winkel - 1Byte

- Datensatz Idencom Compact Format - Minutiaedaten Fix 5 Byte, max. Anzahl 41  
  - X-Position - 1Byte
  - Y-Position - 1Byte
  - W-Winkel - 1Byte
  - X-Position Nachbar- 1Byte
  - Y-Position Nachbar - 1Byte
\enddeutsch

\english  
@b COMMENTARY : the struct of templates is shown you follow

- header fix 11 byte
  - PID(personal identification number) - 8byte / the datafox terminals using only 4 byte
  - FID(finger identification number) - 1byte
  - quality - 1byte
  - minutiae count of template - 1byte
  - records of minutiae

- RECORD DIN-Format - size of minutiae data is fix 3 byte, max. count 50 
  - X-position - 1byte
  - Y-position - 1byte
  - W-angle - 1byte

- RECORD Idencom Compact Format - size of minutiae data is fix 5 byte, max. count 41
  - X-position - 1byte
  - Y-position - 1byte
  - W-angle - 1byte
  - X-position neighbour - 1byte
  - Y-position neighbour - 1byte
\endenglish

  @note \deutsch PID und FID mssen grer Null sein !!! \enddeutsch
        \english IMPORTANT !!! PID and FID should not be NULL \endenglish


@b TEST :     
- 22.07.2008 (MS) \deutsch keine Fehler festgestellt \enddeutsch \english no errors detected \endenglish

*/
/*###########################################################*/

EXTERN_DECL int __stdcall DFBReadTemplate(unsigned char *PID, unsigned char PIDLength,unsigned char FID, unsigned char *Buffer, unsigned int *BufferLength );

/*###########################################################*/
//  FUNCTION:	  DFBFirmwareUpdate
/** \deutsch Fhrt ein Firmwareupdate im Biokey3000 durch \enddeutsch
    \english make a firmware update of module BIOKEY3000 \endenglish


@author Maik Siebrand
@date   14.07.2008

\deutsch   
@b Exportiert:
\enddeutsch
\english
@b Exported:
\endenglish
 _DFBFirmwareUpdate@@4

@param[in] Filename \deutsch Dateiname und Pfad des Updatefiles \enddeutsch
                    \english name and path of update file \endenglish

@return \deutsch Erfolgsmeldung = TRUE, Fehlerfall = FALSE \enddeutsch
        \english no errors give TRUE, if errors occurred give FALSE \endenglish

\deutsch
@b AUFGABE:    
- Liest das Firmwarepaket aus der Datei und bertrgt es in den
  Biokey3000. Die Datei muss vom Typ "*.upd" sein. Nach dem 
  Firmwareupdate muss neu mit den gewnschten Einstellungen 
  initialisiert werden.

@b NDERUNG:
- 19.08.2008 (MS) kleine Anpassungen fr FW-Version 06

@b LAUFZEIT : ca. 20s
\enddeutsch

\english

@b JOB:
- Read the update file and transfer it to the module.
- Be care !!! The update file type is "*.upd".
- After update you have to run DFBInit()

@b MODIFICATIONS:
- 19.08.2008 (MS) little changings for firmware version 6

@b RUNTIME: approximate 20 secounds

\endenglish

@b TEST :     
- 22.07.2008 (MS) \deutsch keine Fehler festgestellt \enddeutsch \english no errors detected \endenglish
- 19.08.2008 (MS) \deutsch keine Fehler festgestellt \enddeutsch \english no errors detected \endenglish

*/
/*###########################################################*/

EXTERN_DECL int __stdcall DFBFirmwareUpdate( char *Filename);

/*############################################################################*/
//  FUNCTION:  DFBFirmwareUpdateSetCallBack
/** \deutsch Setzen einer Rueckruffunktion. \enddeutsch
    \english call back function for fimrware update progress \endenglish


@author Bernd Ottmann
@date   14.07.2008

\deutsch   
@b Exportiert:
\enddeutsch
\english
@b Exported:
\endenglish
 _DFBFirmwareUpdateSetCallBack@@4

@param[in] CBFunction \deutsch Zeiger auf die Rueckruffunktion oder 0 fuer abschalten. \enddeutsch
                      \english pointer of call back function  or NULL if not used \endenglish
\deutsch
@b Aufgaben:
- bergibt den Zeiger auf die Funktion, die den Fortschrittswert fr die 
  Firmwareaktualisierung von DFBFirmwareUpdate() bergibt. 
\enddeutsch

\english
@b JOB:
- Set the pointer of funktion with the progress of firmware update of DFBFirmwareUpdate()
\endenglish  

\deutsch
@b Kommentare:
- Mit der CallBack-Funktion knnen Werte der DLL-Funktionen an Windowsobjekte 
  bergeben werden. In diesem Fall ist es der Abarbeitungsstand beim Firmwareupdate.
\enddeutsch

\english
@b COMMENTARY:
- the call back function is used to give values to windows objects like progress bars
\endenglish  

@b Test:
- 14.07.2008 (MS) \deutsch keine Fehler festgestellt \enddeutsch \english no errors detected \endenglish

\deutsch   
@b BEISPIEL:    
\enddeutsch

\english
@b EXAMPLE:
\endenglish

\deutsch
////Callback Function von CDemo2Dlg benutzt einen Fortschrittbalken
\enddeutsch
\english
////Callback Function of CDemo2Dlg using a progressbar 
\endenglish
int CDemo2Dlg::CBFunction(int pos)  @n
{ @n
  \deutsch   
	//// Nachricht von DLL an Nachrichtenverarbeitung des Dialogs weiterleiten @n
  \enddeutsch
  \english
  //// message from DLL to the dialog progressbar
  \endenglish
	::PostMessage(m_this->GetSafeHwnd(), WM_THREAD, MSG_PROGRESS, pos); @n
	return 1; @n
}

//##################################################################
\deutsch
////ThreadFirmwareupdate der Anwendung mit Benutzung der Callback Function von CDemo2Dlg fr den Fortschrittbalken
\enddeutsch
\english
////ThreadFirmwareupdate of application with using Callback Function of CDemo2Dlg 
\endenglish
int CDemo2App::ThreadFirmwareupdate() @n
{ @n
  int  error=0; @n
  char Version1[6]; @n
  char Version2[6]; @n 
  char message[50]; @n
  char Filename[250]; @n

\deutsch
//// Ausgabe einer Nachricht in einer Textbox
\enddeutsch
\english
//// make a message to the screen
\endenglish

  m_msg.Format("Sie fhren gerade ein Firmwareupdate durch."); @n
	MessageToWindow(MSG_INFORMATION, (long)(LPCTSTR)m_msg); @n

  memset(Version1,0,sizeof(Version1)); @n
  memset(Version2,0,sizeof(Version2)); @n

\deutsch
//// Setzen der Call Back Funktion mit dem Zeicher auf die Variable
\enddeutsch
\english
//// set the call back function with the pointer of variable
\endenglish
  CDemo2Dlg *dlg = (CDemo2Dlg*)m_pDlg; @n
	DFBFirmwareUpdateSetCallBack(dlg->CBFunction); @n

\deutsch
////Dateinamen mit Pfad bergeben @n
\enddeutsch
\english
////get path and filename
\endenglish
  CString str(".\\"); @n
  str = dlg->m_filePath; @n
  strcpy(Filename,str);	 @n
  
\deutsch
//// aktuellen Softwarestand auslesen
\enddeutsch
\english
//// get curently software version
\endenglish
  DFBGetSoftwareVersion(Version1); @n

\deutsch
////Firmwareupdate starten
\enddeutsch
\english
////start update
\endenglish
  if (!DFBFirmwareUpdate(Filename)) error = DFBGetLastErrorNum(); @n

\deutsch
  ////anschlieend den Biokey3000 initialisieren, hier ein paar Beispielwerte  @n
  ////TemplateFormat=Idencom Compact Format, SecurityLevel= 50, Quality=40, MinutiaeMin=12  @n
\enddeutsch
\english
//// now run the initialisation of module
\endenglish
  if (!DFBInit(TEMPLATE_FORMAT,50,40,12)) error = DFBGetLastErrorNum(); @n
  
\deutsch
////Wieder Softwareversion auslesen
\enddeutsch
\english
////get the new softwareversion
\endenglish
  DFBGetSoftwareVersion(Version2); @n
  if (error) @n
  {     @n
    m_msg.Format("Beim Update ist ein Fehler aufgetreten"); @n
	  MessageToWindow(MSG_INFORMATION, (long)(LPCTSTR)m_msg); @n
  }     @n
  else  @n
  {     @n
    sprintf(message,"Sie haben gerade Version %s mit %s ersetzt.",Version1 ,Version2 ); @n
    m_msg.Format(message); @n
	  MessageToWindow(MSG_INFORMATION, (long)(LPCTSTR)m_msg); @n
  }     @n
\deutsch
//// Call Back Funktion beenden
\enddeutsch
\english
//// finish call back function 
\endenglish
  DFBFirmwareUpdateSetCallBack(NULL); @n
	return TRUE; @n
}       @n


*/
/*############################################################################*/

EXTERN_DECL void __stdcall DFBFirmwareUpdateSetCallBack(int (*CBFunction)(int));


/*############################################################################*/
//  FUNCTION:  DFBScanFingerSetCallBack
/** \deutsch Setzen einer Abbruchfunktion bein Scannen. \enddeutsch
    \english call back function for aborting the scann process. \endenglish


@author Bernd Ottmann
@author Maik Siebrand
@date   10.12.2010

\deutsch   
@b Exportiert:
\enddeutsch
\english
@b Exported:
\endenglish
 _DFBScanFingerSetCallBack@@4

@param[in] CBFunction \deutsch Zeiger auf die Rueckruffunktion oder 0 fuer abschalten. \enddeutsch
                      \english pointer of call back function  or NULL if not used \endenglish
\deutsch
@b Aufgaben:
- bergibt den Zeiger auf die Funktion, die prft ob der Scannvorgang bei DFBScanFinger() 
abgebrochen werden soll. Liefert die Funktion TRUE wird weiter gescannt, lieferte die Funktion 
FALSE wird der Scannvorgang beendet.
\enddeutsch

\english
@b JOB:
- Set the pointer of funktion with the return value if DFBScanFinger() have to abort. 
Is the return value of function TRUE scan go on, on FALSE DFBScanFinger() is finished.
\endenglish  

\deutsch
@b Kommentare:
- Mit der CallBack-Funktion knnen Werte der DLL-Funktionen an Windowsobjekte bergeben werden. In diesem Fall ist es eine Abbruchbedingung fr den Scannvorgang.
\enddeutsch

\english
@b COMMENTARY:
- the call back function is used to give values to windows objects like cancel the scann process
\endenglish  

@b Test:
- 10.12.2010 (MS) \deutsch keine Fehler festgestellt \enddeutsch \english no errors detected \endenglish

\deutsch   
@b BEISPIEL:    
\enddeutsch

\english
@b EXAMPLE:
\endenglish

\deutsch
////Callback Function von CDemo2Dlg gibt einen Wert zurck
\enddeutsch
\english
////Callback Function of CDemo2Dlg gives back a value 
\endenglish

       @n
int CDemo2App::ScanFingerCBFunction()       @n
{       @n
       @n
  return m_scanFingerRun;       @n
}       @n


//##################################################################

\deutsch
////ThreadBestFinger ausfhren und Call Back Funktion initialisieren.
\enddeutsch
\english
////ThreadBestFinger start and initialization of Callback Function  
\endenglish

       @n
int CDemo2App::ThreadBestFinger()        @n
{       @n
       @n
  CDlgBestFinger *dlg = (CDlgBestFinger*)m_pDlg;       @n
       @n
\deutsch
//// Setzen auf TRUE damit der Scannvorgang luft.
\enddeutsch
\english
//// set to TRUE that the scan process can runs
\endenglish
  m_scanFingerRun = TRUE;       @n
         @n
\deutsch
//// Setzen der Call Back Funktion mit dem Zeicher auf die Funktion
\enddeutsch
\english
//// set the call back function with the pointer of function
\endenglish
  DFBScanFingerSetCallBack(ScanFingerCBFunction);       @n
       @n
\deutsch
//// Einbinden Test Best Finger Funktion
\enddeutsch
\english
//// call dialog test best finger function
\endenglish
  CallFunctions(dlg->m_fingerCount, dlg->m_fingerScannCount,dlg->m_pid);       @n
       @n
\deutsch
//// Lschen der Call Back Funktion mit dem Zeicher auf die Funktion
\enddeutsch
\english
//// clear the call back function with the pointer of function
\endenglish
  DFBScanFingerSetCallBack(NULL);       @n
       @n
	return TRUE;       @n
}       @n

//##################################################################

\deutsch
////Buttonbefehl abarbeiten
\enddeutsch
\english
////button function work out
\endenglish

       @n
void CDlgBestFinger::OnExecute()        @n
{       @n
  
\deutsch
  ////Prfen ob der Dialog bereits aktiv ist, dann brechen wir den Scannvorgang ab
\enddeutsch
\english
  ////Check if the dialog already active
\endenglish
  if (theApp.m_threadCommand == CDemo2App::BestFinger)       @n
  {       @n
\deutsch
    ////Variable fr den Abbruch setzen
\enddeutsch
\english
    ////set variable FALSE to stop the scann process
\endenglish
    theApp.m_scanFingerRun = FALSE;       @n
    return;       @n
  }       @n
       @n
	if (theApp.m_threadCommand != CDemo2App::idle) {       @n
		       @n
		AfxMessageBox("Es ist noch eine Befehlsausfhrung aktiv.");       @n
		return;       @n
	}       @n
	       @n
	UpdateData(TRUE);       @n
	       @n
	theApp.m_pDlg = this;       @n
	theApp.m_hWnd = GetSafeHwnd();       @n
	theApp.m_threadCommand = CDemo2App::BestFinger;       @n
}       @n

*/
/*############################################################################*/

EXTERN_DECL void __stdcall DFBScanFingerSetCallBack(int (*CBFunction)(void));

/*###########################################################*/
//  FUNCTION:	  DFBGetDatabaseEntries
/** \deutsch Liest die Datenbank aus und speichert die Templates in einer Datei  \enddeutsch
    \english read the template list to a file \endenglish

 @author Maik Siebrand
 @date   30.07.2008 

\deutsch   
@b Exportiert:
\enddeutsch
\english
@b Exported:
\endenglish
 _DFBGetDatabaseEntries@@12

@param[in] Filename \deutsch Dateiname der Templatedatei \enddeutsch
                    \english file name of template file  \endenglish                   
@param[out] EntriesCount \deutsch Anzahl der Templates die gespeichert wurden \enddeutsch
                    \english count of templates in the file  \endenglish                   
                    
@param[in] Flags \deutsch Art der Speicherung Text = 0 oder Binr = 1 \enddeutsch
                    \english typ of template file text = 0 or binary = 1 \endenglish                   
                    
@return \deutsch Erfolgsmeldung = TRUE, Fehlerfall = FALSE \enddeutsch
        \english no errors give TRUE, if errors occurred give FALSE \endenglish

\deutsch   
@b Aufgaben:
- Liest die gesamte Datenbank aus und speichert die Templates als Text oder auch
  binr in einer Datei
\enddeutsch

\english
@b JOB:
- read the template list from biokey module and saves into a file as text or binary format
\endenglish
	
\deutsch @b NDERUNG: \enddeutsch
\english @b MODIFICATIONS: \endenglish

		
@b TEST :     
- 30.07.2008 (MS) \deutsch keine Fehler festgestellt \enddeutsch \english no errors detected \endenglish

*/  
/*###########################################################*/
EXTERN_DECL int __stdcall DFBGetDatabaseEntries( char *Filename, unsigned int *EntriesCount, unsigned char Flags);


#endif


